"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.navigationModel = exports.preserveAnchorCasing = exports.useHTMLAnchors = exports.useHTMLEncodedBrackets = exports.anchorPrefix = exports.publicPath = exports.textContentMappings = exports.sanitizeComments = exports.tableColumnSettings = exports.propertiesFormat = exports.propertyMembersFormat = exports.typeDeclarationFormat = exports.enumMembersFormat = exports.classPropertiesFormat = exports.interfacePropertiesFormat = exports.parametersFormat = exports.indexFormat = exports.blockTagsPreserveOrder = exports.expandParameters = exports.expandObjects = exports.useCodeBlocks = exports.excludeGroups = exports.hideGroupHeadings = exports.hidePageTitle = exports.hideBreadcrumbs = exports.hidePageHeader = exports.mergeReadme = exports.excludeScopesInPaths = exports.entryModule = exports.modulesFileName = exports.entryFileName = exports.fileExtension = exports.flattenOutputFiles = exports.membersWithOwnFile = exports.outputFileStrategy = void 0;
/**
 * Typedoc options declarations.
 *
 * This will be exposed to TypeDoc as a new option when bootstrapping the plugin, with the name of the option the name of the exported variable.
 *
 * The JSDoc comments will also be used in the public facing documentation.
 *
 * @categoryDescription File
 *
 * Options that are used to configure how files are output.
 *
 * @categoryDescription Display
 *
 * Options that are used to configure how the output is structured and displayed.
 *
 * @categoryDescription Utility
 *
 * Options that are used for general configuration and utility purposes.
 *
 * @module
 */
const typedoc_1 = require("typedoc");
const constants_1 = require("./constants");
const maps_1 = require("./maps");
/**
 * TypeDoc creates documentation according to exports derived from the given [`--entryPointsStrategy`](https://typedoc.org/options/input/#entrypointstrategy) TypeDoc configuration.
 *
 * This option provides some flexibility as to how output files are generated.
 *
 * It is also possible to further refine what members are exported to individual files with the `membersWithOwnFile` option.
 *
 * The following keys are available:
 *
 * **"members"**
 *
 * Generates an individual file for each exported module member. This is the standard behavior of the HTML theme and the default setting of the plugin.
 *
 * ```
 *   ├── README.md
 *   ├── module-a/
 *   │   ├── classes/
 *   │   │   ├── ClassA.md
 *   │   │   └── ClassB.md
 *   │   └── functions/
 *   │   │   ├── FunctionA.md
 *   │   │   └── FunctionB.md
 *   └── module-b/
 *       └── classes/
 *           ├── ClassA.md
 *           └── ClassB.md
 * ```
 *
 * **"modules"**
 *
 * Generates a single file for every Module or Namespace where all members are hoisted to a single module file. This creates a flat navigation structure and reduces the amount of files generated.
 *
 * ```
 *   ├── README.md
 *   ├── module-a.md
 *   └── module-b.md
 * ```
 *
 * @category File
 */
exports.outputFileStrategy = {
    help: 'Determines how output files are generated.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.OutputFileStrategy,
    defaultValue: maps_1.OutputFileStrategy.Members,
};
/**
 * This option is useful when only specific types of members should be exported to a single file.
 *
 * Ignored when `--outputFileStrategy` is equal to `"modules"`
 *
 * @example ["Class", "Enum", "Interface"]
 *
 * @category File
 */
exports.membersWithOwnFile = {
    help: 'Determines which members are exported to their own file when `outputFileStrategy` equals `members`.',
    type: typedoc_1.ParameterType.Array,
    validate(values) {
        const validValues = constants_1.ALLOWED_OWN_FILE_MEMBERS;
        for (const kind of values) {
            if (!validValues.includes(kind)) {
                throw new Error(`'${kind}' is an invalid value for 'membersWithOwnFile'. Must be one of: ${validValues.join(', ')}`);
            }
        }
    },
    defaultValue: constants_1.ALLOWED_OWN_FILE_MEMBERS,
};
/**
 * By default output files are generated in a directory structure that mirrors the project's module hierarchy including folders for member kinds eg `classes`, `enums`, `functions` etc.
 *
 * This option will flatten the output files to a single directory as follows:
 *
 * Default output:
 *
 * ```
 *   ├── README.md
 *   ├── module-a/
 *   │   ├── classes/
 *   │   │   ├── ClassA.md
 *   │   │   └── ClassB.md
 *   │   └── functions/
 *   │   │   ├── FunctionA.md
 *   │   │   └── FunctionB.md
 *   └── module-b/
 *       └── classes/
 *           ├── ClassA.md
 *           └── ClassB.md
 * ```
 *
 * Flattened output:
 *
 * ```
 *   ├── README.md
 *   ├── module-a.Class.ClassA.md
 *   ├── module-a.Class.ClassB.md
 *   ├── module-a.Function.functionA.md
 *   ├── module-a.Function.functionB.md
 *   ├── module-b.Class.ClassA.md
 *   └── module-b.Class.ClassB.md
 * ```
 *
 * @category File
 */
exports.flattenOutputFiles = {
    help: 'Flatten output files to a single directory.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * Typically Markdown files are recognised by the `.md` or `.markdown` file extensions.`.mdx` maybe required for compatibility with certain Markdown parsers.
 *
 * @example ".mdx"
 *
 * @category File
 */
exports.fileExtension = {
    help: 'Specify the file extension for generated output files.',
    type: typedoc_1.ParameterType.String,
    defaultValue: '.md',
    validate(value) {
        if (!value.startsWith('.')) {
            throw new Error('[typedoc-plugin-markdown] "fileExtension" must start with a period.');
        }
    },
};
/**
 * The `entryFileName` in this context is the root page of the documentation and each module directory.
 * This is essentially the equivalent to `index.html` for web pages.
 *
 * `README` is recognised when browsing folders on repos and Wikis and is the plugin default. `index` might be more suitable for static site generators.
 *
 * The content of root documentation file will be resolved in the following order:
 *
 * 1. The value of the `--entryModule` option (if defined).
 * 2. The resolved Readme file (skipped if the [`--readme`](https://typedoc.org/options/input/#readme) option is set to `none`).
 * 3. The documentation index page.
 *
 * @example "index"
 *
 * @category File
 *
 */
exports.entryFileName = {
    help: 'The file name of the entry page.',
    type: typedoc_1.ParameterType.String,
    defaultValue: 'README',
};
/**
 * Please note this option is not applicable when [`--readme`](https://typedoc.org/options/input/#readme) is set to "none" or `--mergeReadme` is set to "true".
 *
 * @example "documentation"
 *
 * @defaultValue "modules | packages | globals"
 *
 * @category File
 *
 */
exports.modulesFileName = {
    help: 'The file name of the separate modules / index page.',
    type: typedoc_1.ParameterType.String,
};
/**
 * This option can be used when the root page of the documentation should be a specific module (typically a module named `index`).
 *
 * The module name should be specified (NOT the reference to the file name).
 *
 * Please note a separate modules index page will not be generated, therefore would work better if navigation is present.
 *
 * @example "index"
 *
 * @category File
 */
exports.entryModule = {
    help: 'The name of a module that should act as the root page for the documentation.',
    type: typedoc_1.ParameterType.String,
};
/**
 * By default directories are split by scopes when generating file paths.
 *
 * This option will remove reference to `@scope` in the path when generating files and directories. It does not affect the name of the package or module in the output.
 *
 * The following will be the directory structure for packages named `@scope/package-1` and `@scope/package-2`:
 *
 *  Output when set to `false` (default):
 *
 * ```
 *   └──@scope/
 *       ├── package-1/
 *       └── package-2/
 * ```
 *
 * Output when set to `true`:
 *
 * ```
 *   ├── package-1/
 *   └── package-2/
 * ```
 *
 * Ignored if `flattenOutputFiles` is set to `true`.
 *
 * @category File
 */
exports.excludeScopesInPaths = {
    help: 'Exclude writing @ scope directories in paths.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * By default when a readme file is resolved, a separate readme page is created.
 * This option appends the index page to the readme so only a single root page is generated.
 *
 * This option has no effect when [`--readme`](https://typedoc.org/options/input/#readme) is set to `"none"`.
 *
 * @category File
 */
exports.mergeReadme = {
    help: 'Merges the resolved readme into the project index page.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @category Display
 */
exports.hidePageHeader = {
    help: 'Do not print page header.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @category Display
 */
exports.hideBreadcrumbs = {
    help: 'Do not print breadcrumbs.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @category Display
 */
exports.hidePageTitle = {
    help: 'Do not print page title.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * By default members are grouped by kind (eg Classes, Functions etc).
 *
 * This creates a flat structure where all members are displayed at the same heading level.
 *
 * @category Display
 */
exports.hideGroupHeadings = {
    help: 'Excludes grouping by kind so all members are rendered and sorted at the same level.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @deprecated
 *
 * @hidden
 */
exports.excludeGroups = {
    help: '@deprecated This option has been renamed hideGroupHeadings to better reflect its purpose.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * This option can be used to improve readability and aesthetics when defining signatures and declarations.
 *
 * Please note that when this option is set to `true` it is not possible to link to other references.
 *
 * As a work around the [`@link`](https://typedoc.org/tags/link/) tag can be be used to manually reference types.
 *
 * @category Display
 */
exports.useCodeBlocks = {
    help: 'Wraps signatures and declarations in code blocks.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * By default objects inside declarations are collapsed to preserve space and improve readability.
 *
 * This option should be set when a full object representation is preferred.
 *
 * @category Display
 */
exports.expandObjects = {
    help: 'Expand objects inside declarations.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * By default parameters in signature definitions only display the parameter name so the output is more concise.
 *
 * This option should be set when a full type representation is preferred.
 *
 * @category Display
 */
exports.expandParameters = {
    help: 'Expand parameters in signature parentheses to display type information.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * By default block tags (such as `@example`, `@remarks`, `@deprecated`) are rendered after "Parameters", "Returns" and "Type declaration" sections for signatures and declarations.
 *
 * The rationale is that comment block tags often contain more detailed, supplementary information and are typically secondary to understanding the primary use of the symbol,
 *
 * Use this option to preserve the position of the tag content with the comment summary.
 *
 * @example ["@example", "@deprecated"]
 *
 * @category Display
 */
exports.blockTagsPreserveOrder = {
    help: 'Specifies comment block tags that should preserve their position in relation to the comment summary.',
    type: typedoc_1.ParameterType.Array,
    defaultValue: [],
    validate(value, i18n) {
        if (!value.every((tag) => typeof tag === 'string' && /^@[a-zA-Z][a-zA-Z0-9]*$/.test(tag))) {
            throw new Error(i18n.option_0_values_must_be_array_of_tags('blockTags'));
        }
    },
};
/**
 * This option renders index items either as a simple unordered list or in a table.
 *
 * When table style is selected the following will be the behaviour:
 *
 * - For a **members index**, a description column will be added with key `table` - the first paragraph of the comment summary, or key `htmlTable` - the entire comment contents.
 * - For a **packages index**, (when `--entryPointStrategy` equals `packages`), the package.json description will be displayed with an additional "Version" column (when `--includeVersion` equals true).
 * - For a **documents index** a description column will be added to the table printing the `"description"` frontmatter variable.
 *
 * @category Display
 */
exports.indexFormat = {
    help: 'Sets the format of index items.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * This option specifies the output format for parameters and type parameters of functions and class methods:
 *
 * - **"list"**: parameters are output as bullet points in a linear list, suitable for more detailed comments.
 * - **"table"**: parameters are output within a Markdown table, condensed into a single paragraph.
 * - **"htmlTable"**: parameters are output in an HTML table, enabling block elements to render in table cells.
 *
 * @category Display
 */
exports.parametersFormat = {
    help: 'Sets the format of parameter and type parameter groups.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * This option specifies the output format for interface properties:
 *
 * - **"list"**: properties are output in linear blocks with headings, suitable for more detailed comments.
 * - **"table"**: properties are output within a Markdown table, condensed into a single paragraph.
 * - **"htmlTable"**: properties are output in an HTML table, enabling block elements to render in tabular format.
 *
 * @category Display
 */
exports.interfacePropertiesFormat = {
    help: 'Sets the format of property groups for interfaces.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * This option specifies the output format for class properties:
 *
 * - **"list"**: properties are output in linear blocks with headings, suitable for more detailed comments.
 * - **"table"**: properties are output within a Markdown table, condensed into a single paragraph.
 * - **"htmlTable"**: properties are output in an HTML table, enabling block elements to render in tabular format.
 *
 * @category Display
 */
exports.classPropertiesFormat = {
    help: 'Sets the format of property groups for classes.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * This option specifies the output format for enumeration members:
 *
 * - **"list"**: members are output in linear blocks with headings, suitable for more detailed comments.
 * - **"table"**: members are output within a Markdown table, condensed into a single paragraph.
 * - **"htmlTable"**: members are output in an HTML table, enabling block elements to render in tabular format.
 *
 * @category Display
 */
exports.enumMembersFormat = {
    help: 'Sets the format of enumeration members.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * This option specifies the output format for type declaration of variables and type aliases.
 *
 * - **"list"**: declarations are output in linear blocks with headings, suitable for more detailed comments.
 * - **"table"**: declarations are output within a Markdown table, condensed into a single paragraph.
 * - **"htmlTable"**: declarations are output in an HTML table, enabling block elements to render in tabular format.
 *
 * @category Display
 */
exports.typeDeclarationFormat = {
    help: 'Sets the format of style for type declaration members.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * This option will handle the formatting of object literals assigned as properties in classes or interfaces.
 *
 * Note this options will only take effect when the property declaration is rendered in a `list` format.
 *
 * - **"list"**: members are output in linear blocks with headings, suitable for more detailed comments.
 * - **"table"**: members are output within a Markdown table, condensed into a single paragraph.
 * - **"htmlTable"**: members are output in an HTML table, enabling block elements to render in tabular format.
 *
 * @category Display
 */
exports.propertyMembersFormat = {
    help: 'Sets the format of style for property members for interfaces and classes.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * @hidden
 *
 * @deprecated This option has been deprecated in favour of `--interfacePropertiesFormat` and `--classPropertiesFormat`.
 *
 * @category Display
 */
exports.propertiesFormat = {
    help: 'Sets the format of property groups for interfaces and classes.',
    type: typedoc_1.ParameterType.Map,
    map: maps_1.DisplayFormat,
    defaultValue: maps_1.DisplayFormat.List,
};
/**
 * By default, all available data for symbols are displayed in table columns which can result in several columns in certain use-cases.
 *
 * This option allows you to control the visibility of columns, prioritizing readability over displaying complete data.
 * In addition you can control the alignment of the header text.
 *
 * @category Display
 */
exports.tableColumnSettings = {
    help: 'Control how table columns are configured and displayed.',
    type: typedoc_1.ParameterType.Flags,
    defaults: {
        hideDefaults: false,
        hideInherited: false,
        hideModifiers: false,
        hideOverrides: false,
        hideSources: false,
        hideValues: false,
        leftAlignHeaders: false,
    },
};
/**
 * *Please note this options does not affect the rendering of inline code or code blocks (using single/triple backticks).*
 *
 * By default all comments written inside JsDoc comments will be passed to the output as written, and parsers will interpret un-escaped angle brackets as HTML/JSX tags..
 *
 * This option will escape angle brackets `<` `>` and curly braces `{` `}` written inside JsDoc comments.
 *
 * This option would typically be used when source code comes from an external source exposing the following potential issues:
 *
 * - Comments contain raw tags that should be interpreted as code examples.
 * - Comments contain invalid syntax that (in the case of MDX) will cause breaking parsing errors.
 * - Although most parsers use XSS filters, this option provides an additional layer of XSS security.
 *
 * @category Utility
 */
exports.sanitizeComments = {
    help: 'Sanitize HTML and JSX inside JsDoc comments.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * Defines placeholder text that can be customized. Includes the main page header and breadcrumbs (if displayed),
 * page titles and page footer.
 *
 * Default values within curly braces {} indicates a placeholder of dynamic text.
 *
 * - The `{projectName}` placeholder writes project's name .
 * - The `{kind}` writes the reflection kind of the page.
 * - The `{version}` placeholder writes package version (if includeVersion is `true`).
 *
 * If you are looking for general localization support please see TypeDoc's [`--lang`](https://typedoc.org/options/output/#lang) and [`--locales`](https://typedoc.org/options/output/#locales) options.
 *
 * @category Utility
 */
exports.textContentMappings = {
    help: 'Change specific text placeholders in the template.',
    type: typedoc_1.ParameterType.Object,
    defaultValue: constants_1.TEXT_CONTENT_MAPPINGS,
    validate(value) {
        if (!value || typeof value !== 'object') {
            throw new Error('[typedoc-plugin-markdown] textContentMappings must be an object.');
        }
        for (const val of Object.values(value)) {
            if (typeof val !== 'string') {
                throw new Error(`[typedoc-plugin-markdown] All values of textContentMappings must be strings.`);
            }
        }
    },
};
/**
 * If undefined all urls will be relative.
 *
 * @example "http://abc.com"
 *
 * @category Utility
 */
exports.publicPath = {
    help: 'Specify the base path for all urls.',
    type: typedoc_1.ParameterType.String,
    defaultValue: undefined,
};
/**
 * This option should be used when parsers require a custom anchor prefix.
 *
 * @example "markdown-header"
 *
 * @category Utility
 */
exports.anchorPrefix = {
    help: 'Custom anchor prefix when anchoring to in-page symbols.',
    type: typedoc_1.ParameterType.String,
    defaultValue: undefined,
};
/**
 * By default, opening and closing angle brackets (`<` and `>`) are escaped using backslashes, and most modern Markdown processors handle them consistently.
 * However, using HTML entities (`&lt;` and `&gt;`) might be preferable to avoid any inconsistencies with some Markdown processors.
 *
 * @category Utility
 */
exports.useHTMLEncodedBrackets = {
    help: 'Use HTML encoded entities for angle brackets.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * This option should be used if there are issues when anchoring to symbols within a page.
 *
 * - For Markdown parsers that do not automatically assign header ids.
 * - When cross referencing symbols that are referenced in a table row.
 *
 * @category Utility
 */
exports.useHTMLAnchors = {
    help: 'Add HTML named anchors to headings and table rows.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * By default references to symbol anchor links are lowercased.
 *
 * This option can be used for engines that require the preservation of anchor link casing.
 *
 * @category Utility
 */
exports.preserveAnchorCasing = {
    help: 'Preserve anchor casing when generating link to symbols.',
    type: typedoc_1.ParameterType.Boolean,
    defaultValue: false,
};
/**
 * @deprecated
 *
 * @hidden
 */
exports.navigationModel = {
    help: '@deprecated This option has been deprecated in favour of TypeDoc `navigation` option.',
    type: typedoc_1.ParameterType.Flags,
    defaults: {
        excludeGroups: false,
        excludeCategories: false,
        excludeFolders: false,
    },
};
